@@ -25,15 +25,12 @@ ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #define _GNU_SOURCE
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <locale.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -74,10 +71,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define CAT_NAME            1
 #define CAT_DISP_NAME       2
 
+#define MSG_PULSE  -1
+#define MSG_PROMPT -2
+#define MSG_REBOOT -3
+
 /* Controls */
 
 static GtkWidget *main_dlg, *cat_tv, *pack_tv, *close_btn, *apply_btn, *search_te;
-static GtkWidget *msg_dlg, *msg_msg, *msg_pb, *msg_btn, *msg_cancel, *msg_pbv;
+static GtkWidget *msg_dlg, *msg_msg, *msg_pb, *msg_btn, *msg_cancel;
 static GtkWidget *err_dlg, *err_msg, *err_btn;
 
 /* Data stores for tree views */
@@ -94,32 +95,32 @@ gboolean needs_reboot, first_read, no_update = FALSE, is_pi = TRUE;
 int calls;
 gchar *sel_cat;
 gulong draw_id;
+gboolean wayland = FALSE;
 
 /*----------------------------------------------------------------------------*/
 /* Prototypes                                                                 */
 /*----------------------------------------------------------------------------*/
 
 static char *get_string (char *cmd);
-static char *name_from_id (const gchar *id);
-static void progress (PkProgress *progress, PkProgressType *type, gpointer data);
-static PkResults *error_handler (PkTask *task, GAsyncResult *res, char *desc, gboolean silent, gboolean terminal);
+static void progress (PkProgress *progress, PkProgressType type, gpointer data);
+static PkResults *error_handler (PkTask *task, PkClient *client, GAsyncResult *res, char *desc, gboolean silent, gboolean terminal);
 static gboolean update_self (gpointer data);
 static void refresh_cache_done (PkTask *task, GAsyncResult *res, gpointer data);
 static gboolean filter_fn (PkPackage *package, gpointer user_data);
-static void resolve_1_done (PkTask *task, GAsyncResult *res, gpointer data);
+static void get_updates_done (PkClient *client, GAsyncResult *res, gpointer data);
 static void update_done (PkTask *task, GAsyncResult *res, gpointer data);
-static void read_data_file (PkTask *task);
+static void read_data_file (PkClient *client);
 static gboolean match_pid (char *name, const char *pid);
 static gboolean match_arch (char *arch);
-static void resolve_2_done (PkTask *task, GAsyncResult *res, gpointer data);
-static void details_done (PkTask *task, GAsyncResult *res, gpointer data);
+static void resolve_2_done (PkClient *client, GAsyncResult *res, gpointer data);
+static void details_done (PkClient *client, GAsyncResult *res, gpointer data);
 static void install_handler (GtkButton* btn, gpointer ptr);
 static void install_done (PkTask *task, GAsyncResult *res, gpointer data);
 static void remove_done (PkTask *task, GAsyncResult *res, gpointer data);
 static gboolean reload (GtkButton *button, gpointer data);
 static gboolean quit (GtkButton *button, gpointer data);
 static void error_box (char *msg, gboolean terminal);
-static void message (char *msg, int wait, int prog);
+static void message (char *msg, int type);
 static gboolean clock_synced (void);
 static void resync (void);
 static gboolean ntp_check (gpointer data);
@@ -131,10 +132,110 @@ static gboolean packs_in_cat (GtkTreeModel *model, GtkTreeIter *iter, gpointer d
 static void category_selected (GtkTreeView *tv, gpointer ptr);
 static void install_toggled (GtkCellRendererToggle *cell, gchar *path, gpointer user_data);
 static void close_handler (GtkButton* btn, gpointer ptr);
-static gboolean search_update (GtkEditable *editable, gpointer userdata);
+static void search_update (GtkEditable *editable, gpointer userdata);
 static void get_locales (void);
 static gboolean first_draw (GtkWidget *instance);
 
+#ifdef PK_DEBUG
+const char *roles[PK_ROLE_ENUM_LAST + 1] = {
+	"PK_ROLE_ENUM_UNKNOWN",
+	"PK_ROLE_ENUM_CANCEL",
+	"PK_ROLE_ENUM_DEPENDS_ON",
+	"PK_ROLE_ENUM_GET_DETAILS",
+	"PK_ROLE_ENUM_GET_FILES",
+	"PK_ROLE_ENUM_GET_PACKAGES",
+	"PK_ROLE_ENUM_GET_REPO_LIST",
+	"PK_ROLE_ENUM_REQUIRED_BY",
+	"PK_ROLE_ENUM_GET_UPDATE_DETAIL",
+	"PK_ROLE_ENUM_GET_UPDATES",
+	"PK_ROLE_ENUM_INSTALL_FILES",
+	"PK_ROLE_ENUM_INSTALL_PACKAGES",
+	"PK_ROLE_ENUM_INSTALL_SIGNATURE",
+	"PK_ROLE_ENUM_REFRESH_CACHE",
+	"PK_ROLE_ENUM_REMOVE_PACKAGES",
+	"PK_ROLE_ENUM_REPO_ENABLE",
+	"PK_ROLE_ENUM_REPO_SET_DATA",
+	"PK_ROLE_ENUM_RESOLVE",
+	"PK_ROLE_ENUM_SEARCH_DETAILS",
+	"PK_ROLE_ENUM_SEARCH_FILE",
+	"PK_ROLE_ENUM_SEARCH_GROUP",
+	"PK_ROLE_ENUM_SEARCH_NAME",
+	"PK_ROLE_ENUM_UPDATE_PACKAGES",
+	"PK_ROLE_ENUM_WHAT_PROVIDES",
+	"PK_ROLE_ENUM_ACCEPT_EULA",
+	"PK_ROLE_ENUM_DOWNLOAD_PACKAGES",
+	"PK_ROLE_ENUM_GET_DISTRO_UPGRADES",
+	"PK_ROLE_ENUM_GET_CATEGORIES",
+	"PK_ROLE_ENUM_GET_OLD_TRANSACTIONS",
+	"PK_ROLE_ENUM_REPAIR_SYSTEM",
+	"PK_ROLE_ENUM_GET_DETAILS_LOCAL",
+	"PK_ROLE_ENUM_GET_FILES_LOCAL",
+	"PK_ROLE_ENUM_REPO_REMOVE",
+	"PK_ROLE_ENUM_UPGRADE_SYSTEM",
+	"PK_ROLE_ENUM_LAST"
+};
+
+const char *stati[PK_STATUS_ENUM_LAST + 1] = {
+	"PK_STATUS_ENUM_UNKNOWN",
+	"PK_STATUS_ENUM_WAIT",
+	"PK_STATUS_ENUM_SETUP",
+	"PK_STATUS_ENUM_RUNNING",
+	"PK_STATUS_ENUM_QUERY",
+	"PK_STATUS_ENUM_INFO",
+	"PK_STATUS_ENUM_REMOVE",
+	"PK_STATUS_ENUM_REFRESH_CACHE",
+	"PK_STATUS_ENUM_DOWNLOAD",
+	"PK_STATUS_ENUM_INSTALL",
+	"PK_STATUS_ENUM_UPDATE",
+	"PK_STATUS_ENUM_CLEANUP",
+	"PK_STATUS_ENUM_OBSOLETE",
+	"PK_STATUS_ENUM_DEP_RESOLVE",
+	"PK_STATUS_ENUM_SIG_CHECK",
+	"PK_STATUS_ENUM_TEST_COMMIT",
+	"PK_STATUS_ENUM_COMMIT",
+	"PK_STATUS_ENUM_REQUEST",
+	"PK_STATUS_ENUM_FINISHED",
+	"PK_STATUS_ENUM_CANCEL",
+	"PK_STATUS_ENUM_DOWNLOAD_REPOSITORY",
+	"PK_STATUS_ENUM_DOWNLOAD_PACKAGELIST",
+	"PK_STATUS_ENUM_DOWNLOAD_FILELIST",
+	"PK_STATUS_ENUM_DOWNLOAD_CHANGELOG",
+	"PK_STATUS_ENUM_DOWNLOAD_GROUP",
+	"PK_STATUS_ENUM_DOWNLOAD_UPDATEINFO",
+	"PK_STATUS_ENUM_REPACKAGING",
+	"PK_STATUS_ENUM_LOADING_CACHE",
+	"PK_STATUS_ENUM_SCAN_APPLICATIONS",
+	"PK_STATUS_ENUM_GENERATE_PACKAGE_LIST",
+	"PK_STATUS_ENUM_WAITING_FOR_LOCK",
+	"PK_STATUS_ENUM_WAITING_FOR_AUTH",
+	"PK_STATUS_ENUM_SCAN_PROCESS_LIST",
+	"PK_STATUS_ENUM_CHECK_EXECUTABLE_FILES",
+	"PK_STATUS_ENUM_CHECK_LIBRARIES",
+	"PK_STATUS_ENUM_COPY_FILES",
+	"PK_STATUS_ENUM_RUN_HOOK",
+	"PK_STATUS_ENUM_LAST"
+};
+
+const char *types[PK_PROGRESS_TYPE_SENDER + 1] = {
+	"PK_PROGRESS_TYPE_PACKAGE_ID",
+	"PK_PROGRESS_TYPE_TRANSACTION_ID",
+	"PK_PROGRESS_TYPE_PERCENTAGE",
+	"PK_PROGRESS_TYPE_ALLOW_CANCEL",
+	"PK_PROGRESS_TYPE_STATUS",
+	"PK_PROGRESS_TYPE_ROLE",
+	"PK_PROGRESS_TYPE_CALLER_ACTIVE",
+	"PK_PROGRESS_TYPE_ELAPSED_TIME",
+	"PK_PROGRESS_TYPE_REMAINING_TIME",
+	"PK_PROGRESS_TYPE_SPEED",
+	"PK_PROGRESS_TYPE_DOWNLOAD_SIZE_REMAINING",
+	"PK_PROGRESS_TYPE_UID",
+	"PK_PROGRESS_TYPE_PACKAGE",
+	"PK_PROGRESS_TYPE_ITEM_PROGRESS",
+	"PK_PROGRESS_TYPE_TRANSACTION_FLAGS",
+	"PK_PROGRESS_TYPE_INVALID",
+	"PK_PROGRESS_TYPE_SENDER"
+};
+#endif
 
 /*----------------------------------------------------------------------------*/
 /* Generic helper functions                                                   */
@@ -166,101 +267,75 @@ static char *get_string (char *cmd)
 /* Helper functions for async operations                                      */
 /*----------------------------------------------------------------------------*/
 
-static char *name_from_id (const gchar *id)
+static void progress (PkProgress *progress, PkProgressType type, gpointer data)
 {
-    GtkTreeIter iter;
-    gboolean valid;
-    gchar *tid, *name;
-
-    valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (packages), &iter);
-    while (valid)
-    {
-        gtk_tree_model_get (GTK_TREE_MODEL (packages), &iter, PACK_PACKAGE_ID, &tid, PACK_CELL_NAME, &name, -1);
-        if (!g_strcmp0 (id, tid)) return name;
-        valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (packages), &iter);
-    }
-    return NULL;
-}
-
-static void progress (PkProgress *progress, PkProgressType *type, gpointer data)
-{
-    char *buf, *name;
     int role = pk_progress_get_role (progress);
     int status = pk_progress_get_status (progress);
+    int percent = pk_progress_get_percentage (progress);
 
-    //printf ("progress %d %d %d %d %s\n", role, type, status, pk_progress_get_percentage (progress), pk_progress_get_package_id (progress));
+#ifdef PK_DEBUG
+    printf ("progress %s\t%s\t%s %d %s\n", roles[role], types[type], stati[status], percent, pk_progress_get_package_id (progress));
+#endif
 
     if (msg_dlg)
     {
-        switch (role)
+        if ((type == PK_PROGRESS_TYPE_PERCENTAGE || type == PK_PROGRESS_TYPE_ITEM_PROGRESS
+            || type == PK_PROGRESS_TYPE_PACKAGE || type == PK_PROGRESS_TYPE_PACKAGE_ID
+            || type == PK_PROGRESS_TYPE_DOWNLOAD_SIZE_REMAINING || type == PK_PROGRESS_TYPE_SPEED)
+            && percent >= 0 && percent <= 100)
         {
-            case PK_ROLE_ENUM_REFRESH_CACHE :       if (status == PK_STATUS_ENUM_LOADING_CACHE)
-                                                        message (_("Updating package data - please wait..."), 0, pk_progress_get_percentage (progress));
-                                                    else
-                                                        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
-                                                    break;
-
-            case PK_ROLE_ENUM_RESOLVE :             if (status == PK_STATUS_ENUM_LOADING_CACHE)
-                                                        message (_("Finding packages - please wait..."), 0, pk_progress_get_percentage (progress));
-                                                    else
-                                                        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
-                                                    break;
-
-            case PK_ROLE_ENUM_UPDATE_PACKAGES :     if (status == PK_STATUS_ENUM_LOADING_CACHE)
-                                                        message (_("Updating application - please wait..."), 0, pk_progress_get_percentage (progress));
-                                                    else
-                                                        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
-                                                    break;
-
-            case PK_ROLE_ENUM_GET_DETAILS :         if (status == PK_STATUS_ENUM_LOADING_CACHE)
-                                                        message (_("Reading package details - please wait..."), 0, pk_progress_get_percentage (progress));
-                                                    else
-                                                        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
-                                                    break;
-
-            case PK_ROLE_ENUM_INSTALL_PACKAGES :    if (status == PK_STATUS_ENUM_DOWNLOAD || status == PK_STATUS_ENUM_INSTALL)
-                                                    {
-                                                        name = name_from_id (pk_progress_get_package_id (progress));
-                                                        buf = g_strdup_printf (_("%s %s - please wait..."), status == PK_STATUS_ENUM_INSTALL ? _("Installing") : _("Downloading"),
-                                                            name ? name : _("packages"));
-                                                        message (buf, 0, pk_progress_get_percentage (progress));
-                                                    }
-                                                    else
-                                                        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
-                                                    break;
-
-            case PK_ROLE_ENUM_REMOVE_PACKAGES :     if (status == PK_STATUS_ENUM_REMOVE)
-                                                    {
-                                                        name = name_from_id (pk_progress_get_package_id (progress));
-                                                        if (name)
-                                                        {
-                                                            buf = g_strdup_printf (_("Removing %s - please wait..."), name);
-                                                            message (buf, 0, pk_progress_get_percentage (progress));
-                                                        }
+            switch (role)
+            {
+                case PK_ROLE_ENUM_GET_DETAILS :         if (status == PK_STATUS_ENUM_LOADING_CACHE)
+                                                            message (_("Reading package details - please wait..."), percent);
                                                         else
-                                                            message (_("Removing packages - please wait..."), 0, pk_progress_get_percentage (progress));
-                                                   }
-                                                    else
-                                                        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
-                                                    break;
+                                                            gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
+                                                        break;
+
+                case PK_ROLE_ENUM_UPDATE_PACKAGES :     if (status == PK_STATUS_ENUM_LOADING_CACHE)
+                                                            message (_("Updating application - please wait..."), percent);
+                                                        else
+                                                            gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
+                                                        break;
+
+                case PK_ROLE_ENUM_INSTALL_PACKAGES :    if (status == PK_STATUS_ENUM_DOWNLOAD)
+                                                            message (_("Downloading packages - please wait..."), percent);
+                                                        else if (status == PK_STATUS_ENUM_INSTALL || status == PK_STATUS_ENUM_RUNNING)
+                                                            message (_("Installing packages - please wait..."), percent);
+                                                        else
+                                                            gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
+                                                        break;
+
+                case PK_ROLE_ENUM_REMOVE_PACKAGES :     if (status == PK_STATUS_ENUM_REMOVE || status == PK_STATUS_ENUM_RUNNING)
+                                                            message (_("Removing packages - please wait..."), percent);
+                                                        else
+                                                            gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
+                                                        break;
+
+                default :                               gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
+                                                        break;
+            }
         }
+        else gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
     }
 }
 
-static PkResults *error_handler (PkTask *task, GAsyncResult *res, char *desc, gboolean silent, gboolean terminal)
+static PkResults *error_handler (PkTask *task, PkClient *client, GAsyncResult *res, char *desc, gboolean silent, gboolean terminal)
 {
     PkResults *results;
     PkError *pkerror;
     GError *error = NULL;
     gchar *buf;
 
-    results = pk_task_generic_finish (task, res, &error);
+    if (task) results = pk_task_generic_finish (task, res, &error);
+    else results = pk_client_generic_finish (client, res, &error);
     if (error != NULL)
     {
         if (silent) return NULL;
         buf = g_strdup_printf (_("Error %s - %s"), desc, error->message);
         error_box (buf, terminal);
         g_free (buf);
+        g_error_free (error);
         return NULL;
     }
 
@@ -271,6 +346,7 @@ static PkResults *error_handler (PkTask *task, GAsyncResult *res, char *desc, gb
         buf = g_strdup_printf (_("Error %s - %s"), desc, pk_error_get_details (pkerror));
         error_box (buf, terminal);
         g_free (buf);
+        g_object_unref (pkerror);
         return NULL;
     }
 
@@ -285,27 +361,25 @@ static gboolean update_self (gpointer data)
 {
     PkTask *task;
 
-    message (_("Updating package data - please wait..."), 0 , -1);
+    message (_("Checking for updates - please wait..."), MSG_PULSE);
 
     task = pk_task_new ();
     if (no_update)
     {
-        read_data_file (task);
+        read_data_file (PK_CLIENT (task));
         return FALSE;
     }
-    pk_client_refresh_cache_async (PK_CLIENT (task), TRUE, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) refresh_cache_done, NULL);
+    pk_task_refresh_cache_async (task, TRUE, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) refresh_cache_done, NULL);
     return FALSE;
 }
 
 static void refresh_cache_done (PkTask *task, GAsyncResult *res, gpointer data)
 {
-    gchar *pkg[2] = { "rp-prefapps", NULL };
+    if (!error_handler (task, NULL, res, _("updating package data"), FALSE, TRUE)) return;
 
-    if (!error_handler (task, res, _("updating package data"), FALSE, TRUE)) return;
+    message (_("Checking for updates - please wait..."), MSG_PULSE);
 
-    message (_("Finding packages - please wait..."), 0 , -1);
-
-    pk_client_resolve_async (PK_CLIENT (task), 0, pkg, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) resolve_1_done, NULL);
+    pk_client_get_updates_async (PK_CLIENT (task), 0, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) get_updates_done, NULL);
 }
 
 static gboolean filter_fn (PkPackage *package, gpointer user_data)
@@ -324,14 +398,15 @@ static gboolean filter_fn (PkPackage *package, gpointer user_data)
     return TRUE;
 }
 
-static void resolve_1_done (PkTask *task, GAsyncResult *res, gpointer data)
+static void get_updates_done (PkClient *client, GAsyncResult *res, gpointer data)
 {
     PkResults *results;
     PkPackageSack *sack, *fsack;
     gchar **ids;
     gchar *arch;
+    gchar *todo[2] = {NULL, NULL};
 
-    results = error_handler (task, res, _("finding packages"), TRUE, FALSE);
+    results = error_handler (NULL, client, res, _("finding packages"), TRUE, FALSE);
 
     // Ignore errors here - if the update failed, carry on with existing data...
     if (results)
@@ -342,11 +417,20 @@ static void resolve_1_done (PkTask *task, GAsyncResult *res, gpointer data)
         g_free (arch);
 
         ids = pk_package_sack_get_ids (fsack);
-        if (*ids)
+        int id = 0;
+        while (ids[id])
         {
-            message (_("Updating application - please wait..."), 0 , -1);
-            pk_task_update_packages_async (task, ids, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) update_done, NULL);
-            g_strfreev (ids);
+            if (!strncmp (ids[id], "rp-prefapps;", 12))
+                todo[0] = g_strdup (ids[id]);
+            id++;
+        }
+        g_strfreev (ids);
+
+        if (todo[0])
+        {
+            message (_("Updating application - please wait..."), MSG_PULSE);
+            pk_task_update_packages_async (PK_TASK (client), todo, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) update_done, NULL);
+            g_free (todo[0]);
             g_object_unref (sack);
             g_object_unref (fsack);
         }
@@ -354,28 +438,28 @@ static void resolve_1_done (PkTask *task, GAsyncResult *res, gpointer data)
         {
             g_object_unref (sack);
             g_object_unref (fsack);
-            read_data_file (task);
+            read_data_file (client);
         }
     }
-    else read_data_file (task);
+    else read_data_file (client);
 }
 
 static void update_done (PkTask *task, GAsyncResult *res, gpointer data)
 {
     // No point handling error here - if the update failed, carry on with existing data...
 
-    read_data_file (task);
+    read_data_file (PK_CLIENT (task));
 }
 
-static void read_data_file (PkTask *task)
+static void read_data_file (PkClient *client)
 {
     GtkTreeIter entry, cat_entry;
-    GdkPixbuf *icon;
+    GdkPixbuf *icon = NULL;
     GtkIconInfo *iinfo;
     GKeyFile *kf;
     gchar **groups, **pnames;
     gchar *buf, *cat, *name, *desc, *iname, *loc, *pack, *rpack, *adds, *add, *addspl, *arch;
-    gboolean new, reboot, rpdesc;
+    gboolean new, reboot, rpdesc, xonly, wonly;
     int pcount = 0, gcount = 0;
 
     loc = setlocale (0, "");
@@ -400,6 +484,21 @@ static void read_data_file (PkTask *task)
 
         while (groups[gcount])
         {
+            // skip X-only packages if running under wayland
+            xonly = g_key_file_get_boolean (kf, groups[gcount], "xonly", NULL);
+            if (xonly && wayland)
+            {
+                gcount++;
+                continue;
+            }
+
+            // ... and Wayland-only packages if...oh, you get the idea
+            wonly = g_key_file_get_boolean (kf, groups[gcount], "wonly", NULL);
+            if (wonly && !wayland)
+            {
+                gcount++;
+                continue;
+            }
             cat = g_key_file_get_value (kf, groups[gcount], "category", NULL);
             name = g_key_file_get_value (kf, groups[gcount], "name", NULL);
             desc = g_key_file_get_value (kf, groups[gcount], "description", NULL);
@@ -511,7 +610,11 @@ static void read_data_file (PkTask *task)
                 PACK_ARCH, arch ? arch : "any",
                 PACK_RPDESC, rpdesc,
                 -1);
-            if (icon) g_object_unref (icon);
+            if (icon)
+            {
+                g_object_unref (icon);
+                icon = NULL;
+            }
 
             g_free (buf);
             g_free (cat);
@@ -536,9 +639,9 @@ static void read_data_file (PkTask *task)
         return;
     }
 
-    message (_("Finding packages - please wait..."), 0 , -1);
+    message (_("Reading package details - please wait..."), MSG_PULSE);
 
-    pk_client_resolve_async (PK_CLIENT (task), 0, pnames, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) resolve_2_done, NULL);
+    pk_client_resolve_async (client, 0, pnames, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) resolve_2_done, NULL);
     g_free (pnames);
 }
 
@@ -558,17 +661,28 @@ static gboolean match_pid (char *name, const char *pid)
 
 static gboolean match_arch (char *arch)
 {
+    FILE *fp;
+
     if (!g_strcmp0 (arch, "any")) return TRUE;
 
+    /* check the hardware architecture */
     char *cmd = g_strdup_printf ("arch | grep -q %s", arch);
-    FILE *fp = popen (cmd, "r");
+    fp = popen (cmd, "r");
     int res = pclose (fp);
     g_free (cmd);
-    if (!res) return TRUE;
+    if (res) return FALSE;
+
+    /* additional check for 64-bit only code on ARM */
+    if (!g_strcmp0 (arch, "aarch64"))
+    {
+        fp = popen ("getconf LONG_BIT | grep -q 64", "r");
+        if (!pclose (fp)) return TRUE;
+    }
+
     return FALSE;
 }
 
-static void resolve_2_done (PkTask *task, GAsyncResult *res, gpointer data)
+static void resolve_2_done (PkClient *client, GAsyncResult *res, gpointer data)
 {
     PkResults *results;
     PkPackage *item;
@@ -583,7 +697,7 @@ static void resolve_2_done (PkTask *task, GAsyncResult *res, gpointer data)
     gchar *curr_id;
     int i;
 
-    results = error_handler (task, res, _("finding packages"), FALSE, TRUE);
+    results = error_handler (NULL, client, res, _("finding packages"), FALSE, TRUE);
     if (!results) return;
 
     sack = pk_results_get_package_sack (results);
@@ -712,10 +826,10 @@ static void resolve_2_done (PkTask *task, GAsyncResult *res, gpointer data)
     }
     g_ptr_array_unref (array);
 
-    message (_("Reading package details - please wait..."), 0 , -1);
+    message (_("Reading package details - please wait..."), MSG_PULSE);
 
     ids = pk_package_sack_get_ids (fsack);
-    pk_client_get_details_async (PK_CLIENT (task), ids, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) details_done, NULL);
+    pk_client_get_details_async (client, ids, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) details_done, NULL);
     g_strfreev (ids);
     g_object_unref (sack);
     g_object_unref (fsack);
@@ -739,7 +853,7 @@ static int category_sort (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, g
     return ret;
 }
 
-static void details_done (PkTask *task, GAsyncResult *res, gpointer data)
+static void details_done (PkClient *client, GAsyncResult *res, gpointer data)
 {
     PkResults *results;
     PkDetails *item;
@@ -751,7 +865,7 @@ static void details_done (PkTask *task, GAsyncResult *res, gpointer data)
     const gchar *package_id, *sum, *pd;
     int i;
 
-    results = error_handler (task, res, _("reading package details"), FALSE, TRUE);
+    results = error_handler (NULL, client, res, _("reading package details"), FALSE, TRUE);
     if (!results) return;
 
     array = pk_results_get_details_array (results);
@@ -919,14 +1033,14 @@ static void install_handler (GtkButton* btn, gpointer ptr)
 
     if (n_inst)
     {
-        message (_("Installing packages - please wait..."), 0 , -1);
+        message (_("Downloading packages - please wait..."), MSG_PULSE);
 
         task = pk_task_new ();
         pk_task_install_packages_async (task, pinst, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) install_done, NULL);
     }
     else if (n_uninst)
     {
-        message (_("Removing packages - please wait..."), 0 , -1);
+        message (_("Removing packages - please wait..."), MSG_PULSE);
 
         task = pk_task_new ();
         pk_task_remove_packages_async (task, puninst, TRUE, TRUE, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) remove_done, NULL);
@@ -940,26 +1054,32 @@ static void install_handler (GtkButton* btn, gpointer ptr)
 
 static void install_done (PkTask *task, GAsyncResult *res, gpointer data)
 {
-    if (!error_handler (task, res, _("installing packages"), FALSE, FALSE)) return;
+    if (!error_handler (task, NULL, res, _("installing packages"), FALSE, FALSE)) return;
+
+    // check reboot flag set by install process
+    if (!access ("/run/reboot-required", F_OK)) needs_reboot = TRUE;
 
     if (n_uninst)
     {
-        message (_("Removing packages - please wait..."), 0 , -1);
+        message (_("Removing packages - please wait..."), MSG_PULSE);
 
         pk_task_remove_packages_async (task, puninst, TRUE, TRUE, NULL, (PkProgressCallback) progress, NULL, (GAsyncReadyCallback) remove_done, NULL);
     }
     else
-        message (_("Installation complete"), 1, -1);
+        message (_("Installation complete"), MSG_PROMPT);
 }
 
 static void remove_done (PkTask *task, GAsyncResult *res, gpointer data)
 {
-    if (!error_handler (task, res, _("removing packages"), FALSE, FALSE)) return;
+    if (!error_handler (task, NULL, res, _("removing packages"), FALSE, FALSE)) return;
+
+    // check reboot flag set by install process
+    if (!access ("/run/reboot-required", F_OK)) needs_reboot = TRUE;
 
     if (n_inst)
-        message (_("Installation and removal complete"), 1, -1);
+        message (_("Installation and removal complete"), MSG_PROMPT);
     else
-        message (_("Removal complete"), 1, -1);
+        message (_("Removal complete"), MSG_PROMPT);
 }
 
 static gboolean clock_synced (void)
@@ -1193,13 +1313,13 @@ static gboolean reload (GtkButton *button, gpointer data)
         err_dlg = NULL;
     }
 
-    message (_("Updating package data - please wait..."), 0 , -1);
+    message (_("Checking for updates - please wait..."), MSG_PULSE);
 
     gtk_list_store_clear (packages);
     gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (gtk_tree_view_get_model (GTK_TREE_VIEW (pack_tv))));
     task = pk_task_new ();
     first_read = FALSE;
-    read_data_file (task);
+    read_data_file (PK_CLIENT (task));
     return FALSE;
 }
 
@@ -1216,7 +1336,7 @@ static gboolean quit (GtkButton *button, gpointer data)
         err_dlg = NULL;
     }
 
-    if ((int) data == 1) system ("reboot");
+    if ((long) data == 1) system ("reboot");
 
     gtk_main_quit ();
     return FALSE;
@@ -1250,14 +1370,14 @@ static void error_box (char *msg, gboolean terminal)
         else
             g_signal_connect (err_btn, "clicked", G_CALLBACK (reload), NULL);
 
-        gtk_widget_show_all (err_dlg);
+        gtk_widget_show (err_dlg);
         g_object_unref (builder);
     }
     else gtk_label_set_text (GTK_LABEL (err_msg), msg);
 }
 
 
-static void message (char *msg, int wait, int prog)
+static void message (char *msg, int type)
 {
     if (err_dlg)
     {
@@ -1273,49 +1393,45 @@ static void message (char *msg, int wait, int prog)
         builder = gtk_builder_new_from_file (PACKAGE_DATA_DIR "/rp_prefapps.ui");
 
         msg_dlg = (GtkWidget *) gtk_builder_get_object (builder, "modal");
-        gtk_window_set_transient_for (GTK_WINDOW (msg_dlg), GTK_WINDOW (main_dlg));
-
         msg_msg = (GtkWidget *) gtk_builder_get_object (builder, "modal_msg");
         msg_pb = (GtkWidget *) gtk_builder_get_object (builder, "modal_pb");
         msg_btn = (GtkWidget *) gtk_builder_get_object (builder, "modal_ok");
         msg_cancel = (GtkWidget *) gtk_builder_get_object (builder, "modal_cancel");
 
-        gtk_label_set_text (GTK_LABEL (msg_msg), msg);
+        gtk_window_set_transient_for (GTK_WINDOW (msg_dlg), GTK_WINDOW (main_dlg));
 
         g_object_unref (builder);
     }
-    else gtk_label_set_text (GTK_LABEL (msg_msg), msg);
 
-    if (wait)
+    gtk_label_set_text (GTK_LABEL (msg_msg), msg);
+
+    gtk_widget_hide (msg_pb);
+    gtk_widget_hide (msg_btn);
+    gtk_widget_hide (msg_cancel);
+
+    switch (type)
     {
-        gtk_widget_hide (msg_pb);
-        if (wait > 1)
-        {
-            gtk_button_set_label (GTK_BUTTON (msg_btn), "_Yes");
-            gtk_button_set_label (GTK_BUTTON (msg_cancel), "_No");
-            g_signal_connect (msg_btn, "clicked", G_CALLBACK (quit), (void *) 1);
-            g_signal_connect (msg_cancel, "clicked", G_CALLBACK (quit), (void *) 0);
-            gtk_widget_show (msg_cancel);
-        }
-        else
-        {
-            g_signal_connect (msg_btn, "clicked", G_CALLBACK (reload), NULL);
-            gtk_widget_hide (msg_cancel);
-        }
-        gtk_widget_show (msg_btn);
-    }
-    else
-    {
-        gtk_widget_hide (msg_cancel);
-        gtk_widget_hide (msg_btn);
-        gtk_widget_show (msg_pb);
-        if (prog == -1) gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
-        else
-        {
-            float progress = prog / 100.0;
-            gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (msg_pb), progress);
-        }
+        case MSG_PROMPT:    g_signal_connect (msg_btn, "clicked", G_CALLBACK (reload), NULL);
+                            gtk_widget_show (msg_btn);
+                            break;
+
+        case MSG_REBOOT:    gtk_button_set_label (GTK_BUTTON (msg_btn), "_Yes");
+                            gtk_button_set_label (GTK_BUTTON (msg_cancel), "_No");
+                            g_signal_connect (msg_btn, "clicked", G_CALLBACK (quit), (void *) 1);
+                            g_signal_connect (msg_cancel, "clicked", G_CALLBACK (quit), (void *) 0);
+                            gtk_widget_show (msg_btn);
+                            gtk_widget_show (msg_cancel);
+                            break;
+
+        case MSG_PULSE:     gtk_widget_show (msg_pb);
+                            gtk_progress_bar_pulse (GTK_PROGRESS_BAR (msg_pb));
+                            break;
+
+        default :           gtk_widget_show (msg_pb);
+                            gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (msg_pb), (type / 100.0));
+                            break;
     }
+
     gtk_widget_show (msg_dlg);
 }
 
@@ -1381,12 +1497,12 @@ static void install_toggled (GtkCellRendererToggle *cell, gchar *path, gpointer
 static void close_handler (GtkButton* btn, gpointer ptr)
 {
     if (needs_reboot)
-        message (_("An installed application requires a reboot.\nWould you like to reboot now?"), 2, -1);
+        message (_("An installed application requires a reboot.\nWould you like to reboot now?"), MSG_REBOOT);
     else
         gtk_main_quit ();
 }
 
-static gboolean search_update (GtkEditable *editable, gpointer userdata)
+static void search_update (GtkEditable *editable, gpointer userdata)
 {
     gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (gtk_tree_view_get_model (GTK_TREE_VIEW (pack_tv))));
 }
@@ -1434,7 +1550,7 @@ static gboolean first_draw (GtkWidget *instance)
         if (clock_synced ()) g_idle_add (update_self, NULL);
         else
         {
-            message (_("Synchronising clock - please wait..."), 0, -1);
+            message (_("Synchronising clock - please wait..."), MSG_PULSE);
             calls = 0;
             g_timeout_add_seconds (1, ntp_check, NULL);
         }
@@ -1453,14 +1569,13 @@ int main (int argc, char *argv[])
     GtkBuilder *builder;
     GtkCellRenderer *crp, *crt, *crb;
 
-#ifdef ENABLE_NLS
     setlocale (LC_ALL, "");
     bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
-#endif
 
     if (system ("raspi-config nonint is_pi")) is_pi = FALSE;
+    if (getenv ("WAYLAND_DISPLAY")) wayland = TRUE;
 
     get_locales ();
     needs_reboot = FALSE;
@@ -1505,7 +1620,7 @@ int main (int argc, char *argv[])
     gtk_tree_view_column_set_sizing (gtk_tree_view_get_column (GTK_TREE_VIEW (pack_tv), 0), GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_fixed_width (gtk_tree_view_get_column (GTK_TREE_VIEW (pack_tv), 0), 45);
     gtk_tree_view_column_set_expand (gtk_tree_view_get_column (GTK_TREE_VIEW (pack_tv), 1), TRUE);
-    g_object_set (crt, "wrap-mode", PANGO_WRAP_WORD, "wrap-width", 320, NULL);
+    g_object_set (crt, "wrap-mode", PANGO_WRAP_WORD, "wrap-width", 355, NULL);
     gtk_tree_view_column_set_sizing (gtk_tree_view_get_column (GTK_TREE_VIEW (pack_tv), 2), GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_column_set_fixed_width (gtk_tree_view_get_column (GTK_TREE_VIEW (pack_tv), 2), 50);
 
